<?php

if (!defined('_PS_VERSION_'))
    exit();


use GuzzleHttp\Client;
use GuzzleHttp\Exception\RequestException;
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;
use GuzzleHttp\Event\ErrorEvent;
use GuzzleHttp\Message\Response;

class Pkol extends PaymentModule
{
    private $client;

    public function __construct()
    {
        $this->name = 'pkol';
        $this->tab = 'front_office_features';
        $this->version = '1.0.7';
        $this->author = 'PKO Leasing';
        $this->need_instance = 1;
        $this->ps_versions_compliancy = array('min' => '1.7.1.0', 'max' => _PS_VERSION_);
        $this->bootstrap = true;
        $this->shopID = Configuration::get('shopId');
        $this->secret = Configuration::get('secretkey');
        $this->env = Configuration::get('env');
        $this->controllers = ['validation', 'ajax'];
        $this->widget_style = Configuration::get('widget_style');
        $this->widget_size = Configuration::get('widget_size');
        $this->show_rates = (Configuration::get('widget_rates') == '1' ? 'yes' : 'no');
        $this->show_product = Configuration::get('display_button_0');
        $this->show_cart = Configuration::get('display_button_1');
        $this->show_order = Configuration::get('display_button_2');
        $this->prod_url = 'https://pc.pkoleasing.pl';
        $this->test_url = 'https://stpc.pkoleasing.pl';
        $this->lease_url = '/leasing/init';
        $this->endpoint = '/leasing/public/plugin/simulation';
        $this->app_status = true;
        $this->message = '';
        $this->form = null;
        $this->langID = (int)Configuration::get('PS_LANG_DEFAULT');

        if ($this->env == '1') {
            $this->test_url = $this->prod_url;
            $this->lease_url = $this->prod_url . $this->lease_url;
        } else {
            $this->lease_url = $this->test_url . $this->lease_url;
        }
        if (!$this->shopID || !$this->secret) {
            $this->app_status = false;
        }

        parent::__construct();

        $this->displayName = $this->l('PKO Leasing Online', 'pkol');
        $this->description = $this->l('Finansowanie PKO Leasing - możliwość weryfikacji raty oraz wzięcia leasingu.', 'pkol');

        $this->confirmUninstall = $this->l('Odinstalować wtyczkę?', 'pkol');
    }

    public function install()
    {
        if (Shop::isFeatureActive())
            Shop::setContext(Shop::CONTEXT_ALL);

        return parent::install() &&
            $this->registerHook('displayProductActions') && $this->registerHook('actionFrontControllerSetMedia') && $this->registerHook('paymentOptions') && $this->registerHook('paymentReturn') && $this->registerHook('displayShoppingCartFooter') && $this->registerHook('displayFooterProduct') && $this->registerHook('displayProductAdditionalInfo') && $this->registerHook('displayCartExtraProductActions') && $this->registerHook('displayCrossSellingShoppingCart') && $this->registerHook('displayBackOfficeHeader') && Configuration::updateValue('pkol_shopid', '');
    }

    public function uninstall()
    {
        if (!parent::uninstall() || !Configuration::deleteByName('shopId') || !Configuration::deleteByName('secretkey'))
            return false;
        return true;
    }

    public function validateProduct($data, $type = 'item')
    {
        $settings_token = $this->shopID . ':' . $this->secret;
        $token = base64_encode($settings_token);
        $show_rates = $this->show_rates;
        $check_vat = json_decode($data);
        $taxRate = true;

        foreach ($check_vat->products as $v) {
            if ($v->vatRate == null) {
                $taxRate = false;
            }
        }

        if ($taxRate) {
            $this->client = new \GuzzleHttp\Client([
                'verify' => false,
                'exceptions ' => false,
                'headers' => ['Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $token]
            ]);

            $this->client->getEmitter()->on('error', function (ErrorEvent $event) {
                $event->stopPropagation();
                $event->intercept(new Response(401));
            });

            $response = $this->client->post($this->test_url . $this->endpoint,
                [
                    'headers' => ['Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $token, 'verify' => false],
                    'body' => $data,
                    'timeout' => 3, // Response timeout
                    'connect_timeout' => 3, // Connection timeout,
                    'verify' => false,
                ]
            );

            try {

                if ($response && $response->getStatusCode() == 200) {

                    $data = json_decode($response->getBody());

                    if (!$data->errors && $data->validityResult == 'VALID') {
                        if ($show_rates == 'yes') {
                            $price = $data->firstInstallment->value;
                        }
                        $status = true;
                    } else {
                        $status = false;
                    }

                }
            } catch (RequestException $e) {
                $price = null;
                $status = false;
            }

        } else {
            $status = false;
        }

        if ($type == 'BASKET') {

            $cart = Context::getContext()->cart;
            $rules = $cart->getCartRules();

            if ($cart->getPackageShippingCost() > 0 || !empty($rules)) {
                $price = null;
                $status = false;
            }
        }
        if ($this->widget_style == '0') {
            $color_style = 'blue';
        } else if ($this->widget_style == '1') {
            $color_style = 'white';
        }

        if ($this->widget_size == '0') {
            $size_style = 'xxl';
        } else if ($this->widget_size == '1') {
            $size_style = 'big';
        } else {
            $size_style = 'medium';
        }

        $file = 'pko_' . $size_style . '_' . $color_style . '_leasing.' . ($size_style == 'xxl' ? 'webp' : 'svg');

        if ($size_style == 'big' && $color_style !== 'blue' && $show_rates == 'no') {
            $file = 'pko_' . $size_style . '_' . $color_style . '_leasing.webp';
        }
        $disabled = false;
        $img_styles = '';

        if ($status == false) {
            $show_rates = 'no';
            $file = 'pko_disabled_' . $size_style . '_leasing.' . ($size_style == 'xxl' ? 'webp' : 'svg');
            if ($size_style == 'big' || $size_style == 'xxl') {
                $img_styles = 'border: 1px solid #E6E6E6;border-radius:4px;';
            }
        }
        $text = '';
        $class = '';
        if ($color_style !== 'blue') {
            $border = '';
            $class = 'white';
        }

        if ($show_rates == 'yes' && $price) {
            if ($color_style == 'blue') {
                $color = '#fff';
                $border = '';
            } else {
                $color = '#000';
                $border = '';
            }
            if ($size_style == 'big') {
                $styles = 'style="' . $border . 'position:absolute;right:4px;top:17px;line-height:17px;width:70px;display:inline-block;font-size:14px;font-family:pkobp;color:' . $color . '"';
                $file = str_replace('.svg', '_raty.webp', $file);
            } else if ($size_style == 'xxl') {
                $styles = 'style="' . $border . 'position:absolute;right:5px;top:20px;line-height:17px;width:80px;display:inline-block;font-size:15px;font-family:pkobp;color:' . $color . '"';
                $file = str_replace('.webp', '_raty.webp', $file);
            } elseif ($size_style == 'medium') {
                $styles = 'style="' . $border . 'position: absolute;right: -9px;top: 13px;line-height: 14px;width: 70px;display: inline-block;font-size: 12px;font-family: pkobp;color:' . $color . '"';
                $file = str_replace('.svg', '_raty.webp', $file);
            } else {
                $styles = 'style="' . $border . 'position: absolute;right: -12px;top: 11px;line-height: 12px;width: 60px;display: inline-block;font-size: 11px;font-family: pkobp;color: #000;color:' . $color . '"';
                $file = str_replace('.svg', '_raty.webp', $file);
            }

            $text .= '<span class="pko_rate" ' . $styles . '>Rata od<br/> ' . sprintf('%0.2f', $price) . ' zł</span>';
        }
        $id = 1;

        if ($status === true) {
            $response = '<div class="pkol_widget" ><div data-baseurl="" data-pid="' . $id . '" id="lease_click"  style="display:inline-block;position:relative;">' . $text . '<img class="' . $class . '" src="' . $this->_path . 'views/img/' . $file . '" /></div></div><style>.modal-body .pkol_widget,.modal-body .pko_container,.modal-footer .pkol_widget,.modal-footer .pko_container {display:none!important;} #lease_click:hover {cursor:pointer;}</style>';
            return ['status' => true, 'response' => $response];
        } else {
            $response = $this->_path . 'views/img/' . $file;
            return ['status' => false, 'response' => $response];
        }

    }

    private function getAttributeName($id_lang, $id_att)
    {
        return Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
			SELECT al.*
			FROM ' . _DB_PREFIX_ . 'product_attribute_combination pac
			JOIN ' . _DB_PREFIX_ . 'attribute_lang al ON (pac.id_attribute = al.id_attribute AND al.id_lang=' . (int)$id_lang . ')
			WHERE pac.id_product_attribute=' . (int)$id_att);

    }

    public function getProduct($pid, $att)
    {
        $p = new Product($pid, false, $this->langID);
        $link = new Link();
        $url = $link->getProductLink($pid);

        if ($att > 0) {
            $url = str_replace($pid . '-', $pid . '-' . $att . '-', $url);
        }

        $attrib = $p->getAttributeCombinationsById($att, $this->langID);
        $price_notax = str_replace(',', '', number_format($p->getPriceStatic($pid, false, $att, 2), 2));
        $price_wtax = str_replace(',', '', number_format($p->getPriceStatic($pid, true, $att, 2), 2));
        $name = $p->name;
        $att_data = null;

        foreach ($attrib as $a) {
            $id = $a['id_product_attribute'];
            $att_data = $this->getAttributeName($this->langID, $id);
        }

        if ($att_data) {
            foreach ($att_data as $s) {
                $name .= ' ' . $s['name'];
            }
        }

        return [
            'product_name' => $name,
            'price_with_tax' => $price_wtax,
            'price_without_tax' => $price_notax,
            'url' => $url
        ];

    }

    public function previewCart()
    {
        $data = [];
        $products = Context::getContext()->cart->getProducts();
        $cart = Context::getContext()->cart;

        $product_ids = [];

        if ($this->show_cart !== 'on' || !$this->testConnection()) {
            return '';
        }

        if (empty($products)) {
            return '';
        }

        foreach ($products as $product) {
            $pid = $product['id_product'];

            $quantity = $product['cart_quantity'];
            $p = new Product($pid, false, $this->langID);

            $cat_id = $p->id_category_default;
            $price = $product['specific_prices'];

            $el = [
                'categoryId' => $cat_id,
                'quantity' => $quantity,
                'netValue' => str_replace(',', '', number_format($p->getPriceStatic($pid, false, null, 2), 2)),
                'vatRate' => $this->getRate($p->getTaxesRate())
            ];

            array_push($data, $el);
        }

        $test = [
            'shopId' => $this->shopID,
            'widgetOption' => 1,
            'totalNetValue' => str_replace(',', '', $cart->getOrderTotal(false)),
            'uniqueItemQuantity' => count($products),
            'source' => 'BASKET',
            'products' => $data
        ];

        $data = json_encode($test);

        $check_product = $this->validateProduct($data, 'BASKET');

        // Get Product URL
        if ($product) {
            $link = new Link();
            $url = $link->getProductLink($product);
        }

        $this->smarty->assign('data', $check_product);

        if (!$check_product['status']) {

            $img = '<div class="pko_container"><img src="' . $check_product['response'] . '" /></div>';
            return ['result' => '', 'button' => $img];

        } else {

            $this->checked_product = $check_product;
            $n = 1;
            $type = 'BASKET';
            $url = _PS_BASE_URL_ . __PS_BASE_URI__ . 'koszyk?action=show';
            $form = '<input type="hidden" name="shopId" value="' . $this->shopID . '" />';
            $form .= '<input type="hidden" name="returnLink" value="' . $url . '" />';
            $form .= '<input type="hidden" name="uniqueItemQuantity" value="' . count($products) . '" />';
            $total = 0;
            $total_wt = 0;

            foreach ($products as $product) {

                $pid = $product['id_product'];
                $p = new Product($pid, false, $this->langID);
                $att_id = null;

                if ($product['id_product_attribute'] > 0) {
                    $att_id = $product['id_product_attribute'];
                }

                if ($att_id) {
                    $att = $att_id;
                    $product_data = $this->getProduct($pid, $att_id);
                    $p->name = $product_data['product_name'];
                    $price_without_tax = $product_data['price_without_tax'];
                    $price_with_tax = $product_data['price_with_tax'];
                } else {
                    $att = null;
                    $price_without_tax = str_replace(',', '', number_format($p->getPriceStatic($pid, false, $att, 2), 2));
                    $price_with_tax = str_replace(',', '', number_format($p->getPriceStatic($pid, true, $att, 2), 2));
                }

                $quantity = $product['cart_quantity'];
                $cat_id = $p->id_category_default;
                $price = $product['specific_prices'];
                $p_quant = $quantity;

                $form .= '<input type="hidden" name="productName' . $n . '" value="' . $p->name . '" />';
                $form .= '<input type="hidden" name="productPrice' . $n . '" value="' . $price_with_tax . '" />';
                $form .= '<input type="hidden" name="productNetPrice' . $n . '" value="' . $price_without_tax . '" />';
                $form .= '<input type="hidden" name="productQuantity' . $n . '" value="' . $p_quant . '" />';
                $form .= '<input type="hidden" name="productCategory' . $n . '" value="' . $cat_id . '" />';
                $form .= '<input type="hidden" name="productVatRate' . $n . '" value="' . $this->getRate($p->getTaxesRate()) . '" />';

                $total = $total + $price_without_tax * $p_quant;
                $total_wt = $total_wt + $price_with_tax * $p_quant;
                $n++;
            }

            $form .= '<input type="hidden" name="totalValue" value="' . str_replace(',', '', number_format($cart->getOrderTotal() - $cart->getPackageShippingCost(), 2, '.', '')) . '" />';
            $form .= '<input type="hidden" name="totalNetValue" value="' . str_replace(',', '', number_format((float)$total, 2, '.', '')) . '" />';
            $form .= '<input type="hidden" name="source" value="' . $type . '" />';
            $this->form = $form;

            return ['result' => $form, 'button' => $check_product['response']];
        }
    }

    public function hookDisplayShoppingCartFooter($params)
    {
        $data = [];
        $products = Context::getContext()->cart->getProducts();
        $cart = Context::getContext()->cart;

        $product_ids = [];

        if ($this->show_cart !== 'on' || !$this->testConnection() || empty($products)) {
            return '';
        }

        foreach ($products as $product) {
            $pid = $product['id_product'];
            $p = new Product($pid, false, $this->langID);
            $att_id = null;

            if ($product['id_product_attribute'] > 0) {
                $att_id = $product['id_product_attribute'];
            }

            if ($att_id) {
                $att = $att_id;
                $product_data = $this->getProduct($pid, $att_id);
                $p->name = $product_data['product_name'];
                $price_without_tax = $product_data['price_without_tax'];
                $price_with_tax = $product_data['price_with_tax'];
            } else {
                $att = null;
                $price_without_tax = str_replace(',', '', number_format($p->getPriceStatic($pid, false, $att, 2), 2));
                $price_with_tax = str_replace(',', '', number_format($p->getPriceStatic($pid, false, $att, 2), 2));
            }

            $quantity = $product['cart_quantity'];
            $cat_id = $p->id_category_default;
            $price = $product['specific_prices'];

            $el = [
                'categoryId' => $cat_id,
                'quantity' => $quantity,
                'netValue' => $price_without_tax,
                'vatRate' => $this->getRate($p->getTaxesRate())
            ];
            array_push($data, $el);
        }

        $test = [
            'shopId' => $this->shopID,
            'widgetOption' => 1,
            'totalNetValue' => str_replace(',', '', $cart->getOrderTotal(false)),
            'uniqueItemQuantity' => count($products),
            'source' => 'BASKET',
            'products' => $data
        ];

        $data = json_encode($test);

        $check_product = $this->validateProduct($data, 'BASKET');

        if ($product) {
            $link = new Link();
            $url = $link->getProductLink($product);
        }

        $this->smarty->assign('data', $check_product);
        $this->smarty->assign('form_data', $this->lease_url);

        if (!$check_product['status']) {

            return $this->display(__FILE__, 'button.tpl');
        } else {

            $url = _PS_BASE_URL_ . __PS_BASE_URI__ . 'koszyk?action=show';
            $this->checked_product = $check_product;
            $n = 1;
            $type = 'BASKET';
            $form = '<form id="pkol_form" action="' . $this->lease_url . '" id="leasing" method="POST">';
            $form .= '<input type="hidden" name="shopId" value="' . $this->shopID . '" />';
            $form .= '<input type="hidden" name="returnLink" value="' . $url . '" />';
            $form .= '<input type="hidden" name="uniqueItemQuantity" value="' . count($products) . '" />';
            $total = 0;
            $total_wt = 0;

            foreach ($products as $product) {
                $pid = $product['id_product'];
                $p = new Product($pid, false, $this->langID);
                $att_id = null;

                if ($product['id_product_attribute'] > 0) {
                    $att_id = $product['id_product_attribute'];
                }

                if ($att_id) {
                    $att = $att_id;
                    $product_data = $this->getProduct($pid, $att_id);
                    $p->name = $product_data['product_name'];
                    $price_without_tax = $product_data['price_without_tax'];
                    $price_with_tax = $product_data['price_with_tax'];
                } else {
                    $att = null;
                    $price_without_tax = str_replace(',', '', number_format($p->getPriceStatic($pid, false, $att, 2), 2));
                    $price_with_tax = str_replace(',', '', number_format($p->getPriceStatic($pid, true, $att, 2), 2));
                }

                $quantity = $product['cart_quantity'];
                $cat_id = $p->id_category_default;
                $price = $product['specific_prices'];
                $p_quant = $quantity;

                $form .= '<input type="hidden" name="productName' . $n . '" value="' . $p->name . '" />';
                $form .= '<input type="hidden" name="productPrice' . $n . '" value="' . $price_with_tax . '" />';
                $form .= '<input type="hidden" name="productNetPrice' . $n . '" value="' . $price_without_tax . '" />';
                $form .= '<input type="hidden" name="productQuantity' . $n . '" value="' . $p_quant . '" />';
                $form .= '<input type="hidden" name="productCategory' . $n . '" value="' . $cat_id . '" />';
                $form .= '<input type="hidden" name="productVatRate' . $n . '" value="' . $this->getRate($p->getTaxesRate()) . '" />';

                $total = $total + $price_without_tax * $p_quant;
                $total_wt = $total_wt + $price_with_tax * $p_quant;
                $n++;
            }

            $form .= '<input type="hidden" name="totalValue" value="' . str_replace(',', '', number_format($cart->getOrderTotal() - $cart->getPackageShippingCost(), 2, '.', '')) . '" />';
            $form .= '<input type="hidden" name="totalNetValue" value="' . str_replace(',', '', number_format((float)$total, 2, '.', '')) . '" />';

            $form .= '<input type="hidden" name="source" value="' . $type . '" />';
            $form .= '</form>';

            $form .= '<style>';
            $form .= '.modal-body .pkol_widget,.modal-body .pko_container,.modal-footer .pkol_widget,.modal-footer .pko_container {display:none!important;}';
            $form .= '</style>';
            $form .= '<link rel="stylesheet" href="' . $this->_path . '/views/css/pkol-public.css"></link>';
            $form .= '<input type="hidden" id="endpointurl" name="endpointurl" value="' . $link->getModuleLink('pkol', 'ajax', array()) . '" />';
            $this->form = $form;

            echo $check_product['response'];
            echo $form;
        }
    }

    public function hookDisplayShoppingCart($params)
    {
        return $this->display(__FILE__, 'button.tpl');
    }

    public function hookDisplayCrossSellingShoppingCart($params)
    {
        return '';
    }

    public function getRate($tax)
    {
        $taxes = [
            '0%' => 1,
            '7%' => 2,
            '8%' => 3,
            '22%' => 4,
            '23%' => 5,
            '23' => 5,
            '22' => 4,
            '8' => 3,
            '7' => 2,
            '0' => 1
        ];

        if (isset($taxes[$tax])) {
            return $taxes[$tax];
        } else {
            return null;
        }
    }

    public function hookDisplayProductAdditionalInfo($params)
    {
        global $product;
        if (!$this->app_status) {
            return '';
        }
        if ($this->show_product !== 'on' || !$this->testConnection()) {
            return '';
        }
        if (Configuration::get('display_button_0') !== 'on') {
            return '';
        };

        $pid = Tools::getValue('id_product');
        $att_id = Tools::getValue('id_product_attribute');

        $quantity = 1;
        $product = new Product($pid, false, $this->langID);

        if ($att_id) {
            $att = $att_id;
            $product_data = $this->getProduct($pid, $att_id);
            $product->name = $product_data['product_name'];
            $price_without_tax = $product_data['price_without_tax'];
            $price_with_tax = $product_data['price_with_tax'];
        } else {
            $att = null;
            $price_without_tax = str_replace(',', '', number_format($product->getPriceStatic($pid, false, $att, 2), 2));
            $price_with_tax = str_replace(',', '', number_format($product->getPriceStatic($pid, false, $att, 2), 2));
        }

        if ($product->getQuantity($product->id) == 0) {
            return '';
        }

        $cat_id = $product->id_category_default;
        $test = [
            'shopId' => $this->shopID,
            'widgetOption' => 1,
            'totalNetValue' => $price_without_tax,
            'uniqueItemQuantity' => 1,
            'source' => 'ITEM',
            'products' => [
                [
                    'categoryId' => $cat_id,
                    'quantity' => 1,
                    'netValue' => $price_without_tax,
                    'vatRate' => $this->getRate($product->getTaxesRate())
                ]
            ]
        ];
        $data = json_encode($test);

        $link = new Link();

        $check_product = $this->validateProduct($data);

        $this->smarty->assign('data', $check_product);
        $this->smarty->assign('form_data', '');

        if (!$check_product['status']) {
            return $this->display(__FILE__, 'button.tpl');
        } else {
            $url = $link->getProductLink($product);
            if ($att_id) {
                $url = str_replace($pid . '-', $pid . '-' . $att_id . '-', $url);
            }
            $this->checked_product = $check_product;
            $p_quant = 1;
            $n = 1;
            $type = 'ITEM';
            $form = '<form id="pkol_form" action="' . $this->lease_url . '" id="leasing" method="POST">';
            $form .= '<input type="hidden" name="shopId" value="' . $this->shopID . '" />';
            $form .= '<input type="hidden" name="returnLink" value="' . $url . '" />';
            $form .= '<input type="hidden" name="uniqueItemQuantity" value="1" />';
            $form .= '<input type="hidden" name="productName' . $n . '" value="' . $product->name . '" />';
            $form .= '<input type="hidden" name="productPrice' . $n . '" value="' . str_replace(',', '', number_format($product->getPriceStatic($pid, true, $att, 2), 2)) . '" />';
            $form .= '<input type="hidden" name="productNetPrice' . $n . '" value="' . str_replace(',', '', number_format($product->getPriceStatic($pid, false, $att, 2), 2)) . '" />';
            $form .= '<input type="hidden" name="productQuantity' . $n . '" value="' . $p_quant . '" />';
            $form .= '<input type="hidden" name="productCategory' . $n . '" value="' . $cat_id . '" />';
            $form .= '<input type="hidden" name="productVatRate' . $n . '" value="' . $this->getRate($product->getTaxesRate()) . '" />';
            $form .= '<input type="hidden" name="totalValue" value="' . str_replace(',', '', number_format($product->getPriceStatic($pid, true, $att, 2), 2)) . '" />';
            $form .= '<input type="hidden" name="totalNetValue" value="' . str_replace(',', '', number_format($product->getPriceStatic($pid, false, $att, 2), 2)) . '" />';
            $form .= '<input type="hidden" name="source" value="' . $type . '" />';
            $form .= '</form>';
            $form .= '<link rel="stylesheet" href="' . $this->_path . '/views/css/pkol-public.css"></link>';
            $form .= '<input type="hidden" id="endpointurl" name="endpointurl" value="' . $link->getModuleLink('pkol', 'ajax', array()) . '" />';
            $form .= '<input type="hidden" name="productId" id="productId" value="' . $pid . '" />';
            $form .= '<style>';
            $form .= '.modal-body .pkol_widget,.modal-body .pko_container,.modal-footer .pkol_widget,.modal-footer .pko_container {display:none!important;}';
            $form .= '</style>';

            $this->form = $form;

            echo $check_product['response'];
        }
    }

    public function hookDisplayFooterProduct($params)
    {
        if ($this->form) {
            echo $this->form;
        }
    }

    public function hookDisplayProductActions($params)
    {
    }

    public function hookDisplayCartExtraProductActions($params)
    {
    }

    public function hookProductActions($params)
    {
    }

    public function hookDisplayBackOfficeHeader()
    {
        if (Tools::getValue('configure') != $this->name) {
            return;
        }
        $this->context->controller->addJquery();
        $this->context->controller->addJS($this->_path . 'views/js/custom.js');
    }

    public function verifyConnection()
    {
        $settings_token = $this->shopID . ':' . $this->secret;
        $token = base64_encode($settings_token);

        $this->client = new \GuzzleHttp\Client([
            'verify' => false,
            'exceptions ' => false,
            'headers' => ['Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $token]
        ]);

        try {
            $test = [
                'shopId' => 1,
                'widgetOption' => 2,
                'totalNetValue' => 1000,
                'uniqueItemQuantity' => 1,
                'source' => "BASKET",
                'products' => [
                    [
                        'categoryId' => '475',
                        'quantity' => 1,
                        'netValue' => 1000,
                        'vatRate' => 0
                    ]
                ]
            ];
            $data = json_encode($test);
            $this->client->getEmitter()->on('error', function (ErrorEvent $event) {
                $event->stopPropagation();
                $event->intercept(new Response(401));
            });

            $response = $this->client->post($this->test_url . $this->endpoint,
                [
                    'headers' => ['Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $token, 'verify' => false],
                    'body' => $data,
                    'timeout' => 3, // Response timeout
                    'connect_timeout' => 3, // Connection timeout,
                    'verify' => false,
                ]
            );

            if ($response && $response->getStatusCode() == 200) {
                $this->message = 'Połączenie powiodło się';
                $this->app_status = true;
            }

        } catch (RequestException $e) {
            $this->message = 'Połączenie nie powiodło się sprawdź ponownie konfigurację';
            $this->app_status = false;
        }

        return $this->app_status;
    }

    public function testConnection()
    {
        $settings_token = $this->shopID . ':' . $this->secret;
        $token = base64_encode($settings_token);

        $this->client = new \GuzzleHttp\Client([
            'verify' => false,
            'exceptions ' => false,
            'headers' => ['Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $token, 'verify' => false]
        ]);

        $this->client->getEmitter()->on('error', function (ErrorEvent $event) {
            $event->stopPropagation();
            $event->intercept(new Response(401));
        });

        $test = [
            'shopId' => 1,
            'widgetOption' => 2,
            'totalNetValue' => 1000,
            'uniqueItemQuantity' => 1,
            'source' => "BASKET",
            'products' => [
                [
                    'categoryId' => '475',
                    'quantity' => 1,
                    'netValue' => 1000,
                    'vatRate' => 0
                ]
            ]
        ];

        $data = json_encode($test);
        $this->client->getEmitter()->on('error', function (ErrorEvent $event) {
            $event->stopPropagation();
            $event->intercept(new Response(401));
        });

        $response = $this->client->post($this->test_url . $this->endpoint,
            [
                'headers' => ['Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $token, 'verify' => false],
                'body' => $data,
                'timeout' => 5, // Response timeout
                'connect_timeout' => 5, // Connection timeout,
                'verify' => false,
                'exceptions' => false,
            ]
        );

        try {


            if ($response && $response->getStatusCode() == 200) {
                $this->message = 'Połączenie powiodło się';
                $this->app_status = true;

            } else {
                $this->message = 'Połączenie nie powiodło się sprawdź ponownie konfigurację';
                $this->app_status = false;

            }
        } catch (RequestException $e) {
            $this->message = 'Połączenie nie powiodło się sprawdź ponownie konfigurację';
            $this->app_status = false;


        }
        return $this->app_status;
    }

    public function getContent()
    {
        $output = null;
        $test = (isset($_GET['test']) ? $_GET['test'] : false);

        if ($test && $test == '1') {

            $settings_token = $this->shopID . ':' . $this->secret;
            $token = base64_encode($settings_token);

            $this->client = new \GuzzleHttp\Client([
                'verify' => false,
                'exceptions ' => false,
                'headers' => ['Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $token, 'verify' => false]
            ]);

            $test = [
                'shopId' => 1,
                'widgetOption' => 2,
                'totalNetValue' => 1000,
                'uniqueItemQuantity' => 1,
                'source' => "BASKET",
                'products' => [
                    [
                        'categoryId' => '475',
                        'quantity' => 1,
                        'netValue' => 1000,
                        'vatRate' => 0
                    ]
                ]
            ];

            $data = json_encode($test);

            $this->client->getEmitter()->on('error', function (ErrorEvent $event) {
                $event->stopPropagation();
                $event->intercept(new Response(401));
            });

            $response = $this->client->post($this->test_url . $this->endpoint,
                [
                    'headers' => ['Content-Type' => 'application/json', 'Authorization' => 'Bearer ' . $token, 'verify' => false],
                    'body' => $data,
                    'connect_timeout' => 5, // Connection timeout,
                    'verify' => false,
                    'exceptions' => false,
                ]
            );

            try {
                if ($response && $response->getStatusCode() == 200) {
                    $this->message = 'Połączenie powiodło się';
                    $this->app_status = true;
                    $output .= '<input type="hidden" name="disable_fields" id="disable_fields" value="ok"/>';
                } else {
                    $this->message = 'Połączenie nie powiodło się sprawdź ponownie konfigurację';
                    $this->app_status = false;
                    $output .= '<input type="hidden" name="disable_fields" id="disable_fields" value="1"/>';
                }
            } catch (RequestException $e) {
                $this->message = 'Połączenie nie powiodło się sprawdź ponownie konfigurację';
                $this->app_status = false;
                $output .= '<input type="hidden" name="disable_fields" id="disable_fields" value="1"/>';
            }
        }

        if (Tools::isSubmit('submit' . $this->name)) {
            $this->message = '';

            $shopId = strval(Tools::getValue('shopId'));
            $secretkey = Tools::getValue('secretkey');
            $option0 = Tools::getValue('display_button_0');
            $option1 = Tools::getValue('display_button_1');
            $option2 = Tools::getValue('display_button_2');
            $env = Tools::getValue('env');
            $w_style = Tools::getValue('widget_style');
            $w_size = Tools::getValue('widget_size');
            $w_rates = Tools::getValue('widget_rates');

            if(
                !$shopId
                || empty($shopId)
                || empty($secretkey)
                || !Validate::isGenericName($shopId)
            ) {
                $output .= $this->displayError($this->l('Uzupełnij wymagane pola'));
            } else {
                Configuration::updateValue('shopId', $shopId);
                Configuration::updateValue('secretkey', $secretkey);

                if ($this->verifyConnection()) {
                    Configuration::updateValue('display_button_0', $option0);
                    Configuration::updateValue('display_button_1', $option1);
                    Configuration::updateValue('display_button_2', $option2);
                }

                Configuration::updateValue('env', $env);
                Configuration::updateValue('widget_style', $w_style);
                Configuration::updateValue('widget_size', $w_size);
                Configuration::updateValue('widget_rates', $w_rates);

                $output .= $this->displayConfirmation($this->l('Zmiany zapisane'));
            }
        }

        return $output . $this->displayForm();
    }

    public function getOptions()
    {
        return array(
            array(
                'id_checkbox_options' => 0,
                'checkbox_options_name' => 'Czy wyświetlać przycisk na karcie produktu?'),
            array(
                'id_checkbox_options' => 1,
                'checkbox_options_name' => 'Czy wyświetlać przycisk w koszyku?'),
            array(
                'id_checkbox_options' => 2,
                'checkbox_options_name' => 'Czy wyświetlać metodę płatności w zamówieniu?')
        );
    }

    public function displayForm()
    {
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        $fields_form[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Settings'),
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('ID sklepu'),
                    'name' => 'shopId',
                    'size' => 20,
                    'maxlength' => 10,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Sekretny klucz'),
                    'description' => '',
                    'name' => 'secretkey',
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'checkbox',
                    'label' => 'Wyświetlanie płatności',
                    'desc' => '',
                    'name' => 'display_button',
                    'disabled' => true,
                    'values' => array(
                        'query' => $this->getOptions(),
                        'id' => 'id_checkbox_options',
                        'name' => 'checkbox_options_name',
                        'expand' => array(
                            'print_total' => count($this->getOptions()),
                            'default' => 'show',
                            'show' => array('text' => $this->l('show'), 'icon' => 'plus-sign-alt'),
                            'hide' => array('text' => $this->l('hide'), 'icon' => 'minus-sign-alt')
                        ),
                    ),
                ),
                array(
                    'type' => 'radio',
                    'label' => 'Wybierz środowisko',
                    'name' => 'env',
                    'class' => 't',
                    'required' => true,
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'dev',
                            'value' => 0,
                            'label' => 'Testowe'
                        ),
                        array(
                            'id' => 'prod',
                            'value' => 1,
                            'label' => 'Produkcyjne'
                        )
                    ),
                ),
                array(
                    'type' => 'radio',
                    'label' => 'Wybierz styl widgetu',
                    'name' => 'widget_style',
                    'class' => 't',
                    'required' => true,
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'blue',
                            'value' => 0,
                            'label' => 'Ciemna kolorystyka'
                        ),
                        array(
                            'id' => 'white',
                            'value' => 1,
                            'label' => 'Jasna kolorystyka'
                        )
                    ),
                ),
                array(
                    'type' => 'radio',
                    'label' => 'Wybierz rozmiar widgetu',
                    'name' => 'widget_size',
                    'class' => 't',
                    'required' => true,
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'big',
                            'value' => 0,
                            'label' => 'Duży rozmiar'
                        ),
                        array(
                            'id' => 'medium',
                            'value' => 1,
                            'label' => 'Średni rozmiar'
                        ),
                        array(
                            'id' => 'small',
                            'value' => 2,
                            'label' => 'Mały rozmiar'
                        )
                    ),
                ),
                array(
                    'type' => 'radio',
                    'label' => 'Wyświetl widget: Rata od',
                    'name' => 'widget_rates',
                    'class' => 't',
                    'required' => true,
                    'is_bool' => true,
                    'values' => array(
                        array(
                            'id' => 'v_yes',
                            'value' => 1,
                            'label' => 'Tak'
                        ),
                        array(
                            'id' => 'v_no',
                            'value' => 0,
                            'label' => 'Nie'
                        )
                    ),
                ),

            ),

            'submit' => array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            )
        );

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name;

        // Language
        $helper->default_form_language = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = false;        // false -> remove toolbar
        $helper->toolbar_scroll = false;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit' . $this->name;
        $helper->toolbar_btn = array(
            'save' =>
                array(
                    'desc' => $this->l('Save'),
                    'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                        '&token=' . Tools::getAdminTokenLite('AdminModules'),
                ),
            'back' => array(
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        // Load current value
        $helper->fields_value['shopId'] = Configuration::get('shopId');
        $helper->fields_value['secretkey'] = Configuration::get('secretkey');
        $helper->fields_value['display_button_0'] = Configuration::get('display_button_0');
        $helper->fields_value['display_button_1'] = Configuration::get('display_button_1');
        $helper->fields_value['display_button_2'] = Configuration::get('display_button_2');
        $helper->fields_value['env'] = Configuration::get('env');
        $helper->fields_value['widget_style'] = Configuration::get('widget_style');
        $helper->fields_value['widget_size'] = Configuration::get('widget_size');
        $helper->fields_value['widget_rates'] = Configuration::get('widget_rates');

        return $helper->generateForm($fields_form);
    }

    public function hookPaymentReturn($params)
    {
        if (!$this->active) {
            return;
        }

    }

    public function hookPaymentOptions()
    {
        $products = Context::getContext()->cart->getProducts();
        if ($this->show_order !== 'on' || !$this->testConnection()) {
            return;
        }
        $data = [];

        $cart = Context::getContext()->cart;
        if ($cart->getPackageShippingCost() > 0) {
            return;
        }

        $product_ids = [];

        foreach ($products as $product) {
            $pid = $product['id_product'];
            // var_dump($product);
            $quantity = $product['cart_quantity'];
            $p = new Product($pid, false, $this->langID);

            $cat_id = $p->id_category_default;
            $price = $product['specific_prices'];

            $el = [
                'categoryId' => $cat_id,
                'quantity' => $quantity,
                'netValue' => str_replace(',', '', number_format($p->getPriceStatic($pid, false, null, 2), 2)),
                'vatRate' => $this->getRate($p->getTaxesRate())
            ];
            array_push($data, $el);
        }

        $test = [
            'shopId' => $this->shopID,
            'widgetOption' => 1,
            'totalNetValue' => str_replace(',', '', $cart->getOrderTotal(false)),
            'uniqueItemQuantity' => count($products),
            'source' => 'BASKET',
            'products' => $data
        ];

        $data = json_encode($test);

        $check_product = $this->validateProduct($data, 'BASKET');

        // Get Product URL
        $link = new Link();
        $url = $link->getProductLink($product);

        $this->smarty->assign('data', $check_product);

        if (!$check_product['status']) {
            return;
        } else {
            $newOption = new PaymentOption();
            $newOption->setModuleName($this->name)
                ->setCallToActionText($this->l('PKO Leasing Online'))
                ->setAction($this->context->link->getModuleLink($this->name, 'validation', [], true));
            return [$newOption];
        }
    }

    public function hookActionFrontControllerSetMedia($params)
    {
        // On every pages
        $this->context->controller->registerJavascript(
            'module-modulename-simple-lib',
            'modules/' . $this->name . '/views/js/pkol.js',
            [
                'priority' => 999,
                'attribute' => 'async',
            ]
        );
    }
}
